// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "explosion.h"
#include "gettime.h"
#include <math.h>
#include <stdlib.h>

#ifndef M_PI
#define M_PI 3.1428
#endif // M_PI

Explosion::Explosion(TEXTURE_HANDLE fire0, TEXTURE_HANDLE fire1, TEXTURE_HANDLE smoke0,
    TEXTURE_HANDLE smoke1, vector3 position)
{
    long start_time = GetTime();
    int i;
    Snapshot initial, final;

/*
    initial.size = 0.0f;
    initial.opacity = 0.0f;
    final.size = 4.0f;
    final.opacity = 0.2f;

    for (i = 0; i < 360; i += 40)
    {
        float angle_r = i / (180.0f / M_PI);

        Particle p;
        p.velocity = vector3(18.0f * cos(angle_r), 0.0f, 18.0f * sin(angle_r));
        p.initial_position = position;
        p.maximum_age = 1000 + random()%1000;
        p.flags = 0;
        p.texture = smoke1;
        p.initial = initial;
        p.final = final;

        this->CreateParticle(p, start_time);
    }
*/
    initial.size = 1.0f;
    initial.opacity = 0.0f;
    final.size = 4.0f;
    final.opacity = 0.4f;

    for (i = 5; i < 360; i += 20)
    {
        float angle_r = i / (180.0f / M_PI);

        Particle p;
        p.velocity = vector3(8.0f * cos(angle_r), 0.0f, 8.0f * sin(angle_r));
        p.initial_position = position;
        p.maximum_age = 1000 + rand()%1000;
        p.flags = 0;
        p.mode = SRC_ALPHA_ONE;
        p.texture = smoke0;
        p.initial = initial;
        p.final = final;

        this->CreateParticle(p, start_time);
    }


    initial.size = 0.0f;
    initial.opacity = 1.0f;
    final.size = 8.0f;
    final.opacity = 0.0f;

    for (i = 10; i < 360; i += 80)
    {
        float angle_r = i / (180.0f / M_PI);

        Particle p;
        p.velocity = vector3(3.0f * cos(angle_r), 1.0f, 3.0f * sin(angle_r));
        p.initial_position = position;
        p.maximum_age = 600 + rand()%1000;
        p.flags = 0;
        p.mode = SRC_ALPHA_ONE;
        p.texture = fire1;
        p.initial = initial;
        p.final = final;

        this->CreateParticle(p, start_time);
    }

    initial.size = 0.0f;
    initial.opacity = 1.0f;
    final.size = 3.0f;
    final.opacity = 0.0f;

    for (i = 0; i < 360; i += 80)
    {
        float angle_r = i / (180.0f / M_PI);

        Particle p;
        p.velocity = vector3(1.0f * cos(angle_r), 1.0f, 1.0f * sin(angle_r));
        p.initial_position = position;
        p.maximum_age = 200 + rand()%1000;
        p.flags = 0;
        p.mode = SRC_ALPHA_ONE;
        p.texture = fire0;
        p.initial = initial;
        p.final = final;

        this->CreateParticle(p, start_time);
    }
}

Explosion::~Explosion()
{

}

void Explosion::Update()
{
    this->UpdateParticles();
}

void Explosion::Render()
{
    this->SubmitParticlesToParticleRenderer();
}
