// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#ifndef FLARE_H
#define FLARE_H

#include "particle_system.h"

//! Flare inherits from base class particle system
class Flare : public ParticleSystem
{
public:
    // Creates a flare. A flare is sort of similar to a smoke trail but with a fire texture. 
    // fire0 is the texture to use for the flare.
    // initial_position is the point at which the trail started usually the rocket laucher's
    // location at that time.
    // velocity is the speed of the fire source in unit's per second.
    Flare(TEXTURE_HANDLE fire0, vector3 initial_position, vector3 velocity);

    // For best results this shouldn't be destructed until after the fire has faded completely.
    ~Flare();

    // Call this while the rocket is still emitting fire.
    void Update();

    // Call this to render the particles.
    void Render();

private:

    // Initital position and velocity. velocity is always constant.
    vector3 pos, v;

    // Texture used by fire particles
    TEXTURE_HANDLE fire;
};

#endif // FLARE_H
