// Slickworm
// Copyright (c) 2002-2003, Michel O'Brien.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  1. Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//  2. Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//  3. Neither the names of the copyright holders nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "landscape_deformation.h"
#include "get_distance.h"
#include <iostream>

void sphere_subtract(Landscape* land, update_rect& rect, vector3 centre, float radius)
{
    // Calculate the rectangular region that needs to be modified
    float top_f     = centre.z - radius;
    float bottom_f  = centre.z + radius;
    float left_f    = centre.x - radius;
    float right_f   = centre.x + radius;

    int top     = (int)floor(top_f);
    int bottom  = (int)ceil(bottom_f);
    int left    = (int)floor(left_f);
    int right   = (int)ceil(right_f);

    int update_top      = 513;
    int update_bottom   = 0;
    int update_left     = 513;
    int update_right    = 0;

    float* terrain_f = land->GetHeightfieldF();

    for (int x = left; x <= right; x++)
    {
        for (int y = top; y <= bottom; y++)
        {
            if (x >= 0 && x < 513 && y >= 0 && y < 513)
            {
                if (get_distance(centre, (float)x, centre.y, (float)y) < radius)
                {
                    if (centre.y < terrain_f[x + y * 513])
                    {
                        terrain_f[x + y * 513] = centre.y;
                    }

                    if (x < update_left)    update_left     = x;
                    if (x > update_right)   update_right    = x;
                    if (y < update_top)     update_top      = y;
                    if (y > update_bottom)  update_bottom   = y;
                }

                if (get_distance(centre, (float)x, terrain_f[x + y * 513], (float)y) < radius)
                {
                    while ((get_distance(centre, (float)x, terrain_f[x + y * 513],
                        (float)y) < radius) && terrain_f[x + y * 513] > 0.1f)
                    {
                        terrain_f[x + y * 513]-= 0.10f;
                    }

                    if (x < update_left)    update_left     = x;
                    if (x > update_right)   update_right    = x;
                    if (y < update_top)     update_top      = y;
                    if (y > update_bottom)  update_bottom   = y;
                }
            }
        }
    }

    if (update_left > update_right || update_top > update_bottom)
    {
        update_top      = 0;
        update_bottom   = 0;
        update_left     = 0;
        update_right    = 0;
    }

    rect.top    = update_top;
    rect.bottom = update_bottom;
    rect.left   = update_left;
    rect.right  = update_right;
}

void sphere_add(Landscape* land, update_rect& rect, vector3 centre, float radius)
{
    // TODO: This function
}

